﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace WindowsFormsApp10
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private int[,] CreateHistogram(Bitmap bmap)
        {
            int i, j;
            Color color1;
            byte r, g, b;
            int[,] his = new int[4, 256];  //戻り値を入れる配列

            //配列を0クリアする
            for (i = 0; i < 4; i++)
                for (j = 0; j < 256; j++)
                {
                    his[i, j] = 0;

                }
            //原画像のヒストグラムを採る
            for (j = 0; j < bmap.Height; j++)
                for (i = 0; i < bmap.Width; i++)
                {
                    color1 = bmap.GetPixel(i, j);
                    r = color1.R;
                    g = color1.G;
                    b = color1.B;
                    his[0, r]++;
                    his[1, g]++;
                    his[2, b]++;
                }
            //his[0,],his[1,i,his[2,i]の最大値を求めてhis[3,i]とする
            for (i = 0; i < 256; i++)
                his[3, i] = Math.Max(Math.Max(his[0, i], his[1, i]), his[2, i]);
            return his;
        }

        private Bitmap StretchHistogram(Bitmap bmap, byte low, byte high)
        {
            int i, j;
            Color color1;
            byte r, g, b, r1, g1, b1;
            Bitmap bmap1 = new Bitmap(bmap.Width, bmap.Height);
            float p = 255F / (high - low);

            for (j = 0; j < bmap.Height; j++)
                for (i = 0; i < bmap.Width; i++)
                {
                    color1 = bmap.GetPixel(i, j);
                    r = color1.R;
                    g = color1.G;
                    b = color1.B;
                    if (r < low) r1 = 0;
                    else if (r > high) r1 = 255;
                    else r1 = (byte)(p * (r - low));
                    if (g < low) g1 = 0;
                    else if (g > high) g1 = 255;
                    else g1 = (byte)(p * (g - low));
                    if (b < low) b1 = 0;
                    else if (b > high) b1 = 255;
                    else b1 = (byte)(p * (b - low));
                    color1 = Color.FromArgb(r1, g1, b1);
                    bmap1.SetPixel(i, j, color1);
                }
            return bmap1;
        }

        private Bitmap EqualizeHistogram(Bitmap bmap, int[,] his)
        {
            int i, j;
            Color color1;
            byte r, g, b, r1, g1, b1, d;
            Bitmap bmap1 = new Bitmap(bmap.Width, bmap.Height);
            int sum, integral, average;
            byte[] table = new byte[256];

            //明度ヒストグラムのデータ数の合計を求める
            sum = 0;
            for (i = 0; i < 256; i++)
                sum += his[3, i];
            //明度ヒストグラムのデータ数の平均を求める（端数は切り上げ）
            average = (int)Math.Ceiling(sum / 256.0);
            //変換テーブルを求める
            integral = 0;
            j = 0;
            for (i = 0; i < 256; i++)
            {
                integral += his[3, i];
                j += integral / average;  //整数除算商
                integral %= average;      //整数除算剰余
                table[i] = (byte)j;
            }
            //変換テーブルを用いてRGB各ヒストグラムを補正する
            for (j = 0; j < bmap.Height; j++)
                for (i = 0; i < bmap.Width; i++)
                {
                    color1 = bmap.GetPixel(i, j);
                    r = color1.R;
                    g = color1.G;
                    b = color1.B;
                    r1 = table[r];
                    g1 = table[g];
                    b1 = table[b];
                    color1 = Color.FromArgb(r1, g1, b1);
                    bmap1.SetPixel(i, j, color1);
                }
            return bmap1;
        }

        private void DrawHistogram(int x, int y, int[,] his)
        {
            Graphics gr = CreateGraphics();
            int i;
            double scale;

            //ヒストグラムhis[3,]の最大値を求める
            int maximum = 0;
            for (i = 1; i <= 254; i++)   //0と255を除く
                if (his[3, i] > maximum) maximum = his[3, i];
            //最大値から、表示のスケーリングを行う
            scale = 50.0 / maximum;
            //ヒストグラム表示の枠を描く
            for (i = 0; i < 4; i++)
                gr.DrawRectangle(Pens.Black, x, y + 60 * i, 256, 60);
            //ヒストグラムを表示する
            for (i = 1; i <= 254; i++)   //0と255は表示しない
            {
                gr.DrawLine(Pens.Red, x + i, y + 59, x + i, y + 60 - (int)(his[0, i] * scale));
                gr.DrawLine(Pens.Green, x + i, y + 119, x + i, y + 120 - (int)(his[1, i] * scale));
                gr.DrawLine(Pens.Blue, x + i, y + 179, x + i, y + 180 - (int)(his[2, i] * scale));
                gr.DrawLine(Pens.Gray, x + i, y + 239, x + i, y + 240 - (int)(his[3, i] * scale));
            }
        }

        //DrawHistogramのオーバーロード（多重定義）で、5%と95%を表示する
        private void DrawHistogram(int x, int y, int[,] his, byte low, byte high)
        {
            Graphics gr = CreateGraphics();
            int i;
            float scale;

            //ヒストグラムhis[3,i]の最大値を求める
            int maximum = 0;
            for (i = 1; i <= 254; i++)   //0と255を除く
                if (his[3, i] > maximum) maximum = his[3, i];
            //最大値から、表示のスケーリングを行う
            scale = 50F / maximum;
            //ヒストグラム表示の枠を描く
            for (i = 0; i < 4; i++)
                gr.DrawRectangle(Pens.Black, x, y + 60 * i, 256, 60);
            //ヒストグラムを表示する
            for (i = 1; i <= 254; i++)   //0と255は表示しない
            {
                gr.DrawLine(Pens.Red, x + i, y + 59, x + i, y + 60 - (int)(his[0, i] * scale));
                gr.DrawLine(Pens.Green, x + i, y + 119, x + i, y + 120 - (int)(his[1, i] * scale));
                gr.DrawLine(Pens.Blue, x + i, y + 179, x + i, y + 180 - (int)(his[2, i] * scale));
                gr.DrawLine(Pens.Gray, x + i, y + 239, x + i, y + 240 - (int)(his[3, i] * scale));
            }
            //5%と95%を表示する
            gr.DrawLine(Pens.Black, x + low, y, x + low, y + 240);
            gr.DrawLine(Pens.Black, x + high, y, x + high, y + 240);
        }

        private void Form1_Paint(object sender, PaintEventArgs e)
        {
            Graphics gr = e.Graphics;
            int HEIGHT = 240, WIDTH = 320;   //画像のサイズ
            int X0 = 10, Y0 = 10;
            int X1 = 350, Y1 = 260;
            byte d, level05, level95;
            int integral;
            Bitmap bmap1 = new Bitmap(WIDTH, HEIGHT);
            int[,] hist = new int[4, 256];  //ヒストグラムを入れる二次元配列

            //原画像を読み込んでbmapを得る
            Bitmap bmap = new Bitmap("budapest.jpg");  //sample.jpgも使用できる
            //原画像を左上に表示する
            gr.DrawImage(bmap, X0, Y0, WIDTH, HEIGHT);
            //原画像のヒストグラムを採る
            hist = CreateHistogram(bmap);
            //明度ヒストグラムの5%を求める
            integral = 0;
            d = 0;           //濃度0から始めて増やしてゆく
            do
            {
                integral += hist[3, d];
                d++;
            } while (integral < 3840);    //3840 = 総ピクセル数 x 5%
            level05 = (byte)(d - 1);
            //明度ヒストグラムの95%を求める
            integral = 0;
            d = 255;          //濃度255から始めて減らしてゆく
            do
            {
                integral += hist[3, d];
                d--;
            } while (integral < 3840);    //3840 = 総ピクセル数 x 5%
            level95 = (byte)(d + 1);

            //------- ヒストグラム拡張の場合はこちらを使う -------
            //ヒストグラムを表示する（Leve05とLevel95あり）
            DrawHistogram(X1, Y0, hist, level05, level95);
            //ヒストグラムを拡張する
            bmap1 = StretchHistogram(bmap, level05, level95);
            //----------------------------------------------------

            //------ ヒストグラム平坦化の場合はこちらを使う ------
            ////ヒストグラムを表示する（Leve05とLevel95なし）
            //DrawHistogram(X1, Y0, hist);
            ////ヒストグラムを平坦化する
            //bmap1 = EqualizeHistogram(bmap, hist);
            //----------------------------------------------------

            //ヒストグラム補正画像を左下に表示する
            gr.DrawImage(bmap1, X0, Y1, WIDTH, HEIGHT);
            //補正画像のヒストグラムを採る
            hist = CreateHistogram(bmap1);
            //ヒストグラムを表示する（Leve05とLevel95なし）
            DrawHistogram(X1, Y1, hist);
        }
    }
}
