﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace WindowsFormsApp11
{
    public partial class Form1 : Form
    {
        bool first_flag = true;
        const int HEIGHT = 240, WIDTH = 320;
        float[,] hue = new float[WIDTH, HEIGHT];
        float[,] saturation = new float[WIDTH, HEIGHT];
        float[,] value = new float[WIDTH, HEIGHT];
        Bitmap bmap = new Bitmap(WIDTH, HEIGHT);

        public Form1()
        {
            InitializeComponent();
        }

        private float[] ConvertToHSV(int r, int g, int b)
        {
            float[] hhssvv = new float[3];
            float hue = 0, saturation, value;
            int max = Math.Max(r, Math.Max(g, b));
            int min = Math.Min(r, Math.Min(g, b));
            int diff = max - min;
            if (diff != 0)
            {
                if (max == r) hue = 60 * (g - b) / (float)diff;
                if (max == g) hue = 60 * (b - r) / (float)diff + 120;
                if (max == b) hue = 60 * (r - g) / (float)diff + 240;
            }
            else hue = 0;
            if (max != 0) saturation = diff / (float)max;
            else saturation = 0;
            value = max / 255F;
            hhssvv[0] = hue;
            hhssvv[1] = saturation;
            hhssvv[2] = value;
            return hhssvv;
        }

        private Color CreateColorFromHSV(float h, float s, float v)
        {
            int r = 0, g = 0, b = 0;   //仮設定
            int h6;              //色相環を6分割した番号0～5
            float f;             //6分割の中での位置0～1
            int high, low, up, down;

            if (h >= 360) h -= 360;
            else if (h < 0) h += 360;
            high = (int)(255 * v);
            if (high > 255) high = 255;
            else if (high < 0) high = 0;
            if (s == 0)
            {
                r = high; g = high; b = high;
            }
            else
            {
                h6 = (int)(h / 60);   //除算で小数点以下切り捨て
                f = h / 60 - h6;      //除算の小数点以下の部分（fraction）
                low = (int)(high * (1 - s));
                down = (int)(high * (1 - f * s));
                up = (int)(high * (1 - (1 - f) * s));
                if (low > 255) low = 255;
                else if (low < 0) low = 0;
                if (down > 255) down = 255;
                else if (down < 0) down = 0;
                if (up > 255) up = 255;
                else if (up < 0) up = 0;
                switch (h6)
                {
                    case 0: r = high; g = up; b = low; break;
                    case 1: r = down; g = high; b = low; break;
                    case 2: r = low; g = high; b = up; break;
                    case 3: r = low; g = down; b = high; break;
                    case 4: r = up; g = low; b = high; break;
                    case 5: r = high; g = low; b = down; break;
                }
            }
            return Color.FromArgb(r, g, b);
        }

        private void Form1_Paint(object sender, PaintEventArgs e)
        {
            Graphics gr = e.Graphics;
            int X0 = 10, Y0 = 10;
            int X1 = 10, Y1 = HEIGHT + 20;
            Bitmap bmap1 = new Bitmap(WIDTH, HEIGHT);
            Color color1;
            int r, g, b;
            float h, s, v;
            float[] hsv = new float[3];
            float hueRotate, saturationMult, valueMult;

            if (first_flag)
            {
                //原画像を読み込んでbmapを得る
                bmap = new Bitmap("tower.jpg");
                //原画像をRGBに分解し、HSVに変換して配列に入れる
                for (int j = 0; j < HEIGHT; j++)
                    for (int i = 0; i < WIDTH; i++)
                    {
                        color1 = bmap.GetPixel(i, j);
                        r = color1.R;
                        g = color1.G;
                        b = color1.B;
                        hsv = ConvertToHSV(r, g, b);
                        hue[i, j] = hsv[0];
                        saturation[i, j] = hsv[1];
                        value[i, j] = hsv[2];
                    }
                first_flag = false;
            }
            //スクロールバーを読み取り、値を表示する
            hueRotate = hScrollBar1.Value;
            saturationMult = hScrollBar2.Value / 100F;
            valueMult = hScrollBar3.Value / 100F;
            label10.Text = String.Format("{0} °", hScrollBar1.Value);
            label11.Text = String.Format("{0} %", hScrollBar2.Value - 100);
            label12.Text = String.Format("{0} %", hScrollBar3.Value - 100);
            //原画像を左上に表示する
            gr.DrawImage(bmap, X0, Y0, WIDTH, HEIGHT);
            //原画像のh,s,vを配列から読み出し、補正する
            for (int j = 0; j < HEIGHT; j++)
                for (int i = 0; i < WIDTH; i++)
                {
                    h = hue[i, j] + hueRotate;
                    s = saturation[i, j] * saturationMult;
                    v = value[i, j] * valueMult;
                    color1 = CreateColorFromHSV(h, s, v);
                    bmap1.SetPixel(i, j, color1);
                }
            //補正画像を左下に表示する
            gr.DrawImage(bmap1, X1, Y1, WIDTH, HEIGHT);
        }

        private void hScrollBar1_Scroll(object sender, ScrollEventArgs e)
        {
            Invalidate();
        }

        private void hScrollBar2_Scroll(object sender, ScrollEventArgs e)
        {
            Invalidate();
        }

        private void hScrollBar3_Scroll(object sender, ScrollEventArgs e)
        {
            Invalidate();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            hScrollBar1.Value = 0;
            hScrollBar2.Value = 100;
            hScrollBar3.Value = 100;
            Invalidate();
        }
    }
}
