﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace WindowsFormsApp12
{
    public partial class Form1 : Form
    {
        const int WIDTH = 320, HEIGHT = 240;
        Bitmap bmap = new Bitmap(WIDTH, HEIGHT);       //原画像を入れる
        Color[,] pdata = new Color[WIDTH, HEIGHT];     //原画像ピクセルのカラー情報
        float[,] kernel = new float[3, 3];             //一般カーネル
        float[,] kernel_blur = {{0.11F,0.11F,0.11F},   //平滑化カーネル                                   
                                {0.11F,0.12F,0.11F},
                                {0.11F,0.11F,0.11F}};
        float[,] kernel_sharp = {{ 0.0F,-0.3F, 0.0F},  //鮮鋭化カーネル                                   
                                 {-0.3F, 2.2F,-0.3F},
                                 { 0.0F,-0.3F, 0.0F}};

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            //原画像を読み込む
            bmap = new Bitmap("belgium.jpg");
            //原画像を二次元ピクセルデータpdataに変換する
            for (int j = 0; j < HEIGHT; j++)
                for (int i = 0; i < WIDTH; i++)
                {
                    pdata[i, j] = bmap.GetPixel(i, j);
                }
        }

        private void Form1_Paint(object sender, PaintEventArgs e)
        {
            Graphics gr = e.Graphics;
            //原画像を左上に表示する
            gr.DrawImage(bmap, 10, 10, WIDTH, HEIGHT);
        }

        private Bitmap MakeConvolution(Color[,] pdata, float[,] kernel)
        {
            int i, j, m, n;
            float r1, g1, b1;    //コンボリューション結果
            int r, g, b;         //上記をbyte型範囲に収めたRGB値
            Color color1;
            Bitmap bmap1 = new Bitmap(WIDTH, HEIGHT);  //処理後画像

            //原画像の周辺をそのままコピーする
            for (i = 0; i < WIDTH; i++)
            {
                color1 = bmap.GetPixel(i, 0);
                bmap1.SetPixel(i, 0, color1);
                color1 = bmap.GetPixel(i, HEIGHT - 1);
                bmap1.SetPixel(i, HEIGHT - 1, color1);
            }
            for (j = 0; j < HEIGHT; j++)
            {
                color1 = bmap.GetPixel(0, j);
                bmap1.SetPixel(0, j, color1);
                color1 = bmap.GetPixel(WIDTH - 1, j);
                bmap1.SetPixel(WIDTH - 1, j, color1);
            }
            //コンボリューションの実施
            for (j = 1; j < HEIGHT - 1; j++)
                for (i = 1; i < WIDTH - 1; i++)
                {
                    r1 = 0F; g1 = 0F; b1 = 0F;
                    for (n = -1; n <= 1; n++)
                        for (m = -1; m <= 1; m++)
                        {
                            r1 += pdata[i + m, j + n].R * kernel[m + 1, n + 1];
                            g1 += pdata[i + m, j + n].G * kernel[m + 1, n + 1];
                            b1 += pdata[i + m, j + n].B * kernel[m + 1, n + 1];
                        }
                    r = (int)r1;
                    if (r < 0) r = 0;
                    else if (r > 255) r = 255;
                    g = (int)g1;
                    if (g < 0) g = 0;
                    else if (g > 255) g = 255;
                    b = (int)b1;
                    if (b < 0) b = 0;
                    else if (b > 255) b = 255;
                    bmap1.SetPixel(i, j, Color.FromArgb(r, g, b));
                }
            return bmap1;
        }

        private void buttonFilter_Click(object sender, EventArgs e)
        {
            textBox1.Text = Convert.ToString(kernel_blur[0, 0]);
            textBox2.Text = Convert.ToString(kernel_blur[1, 0]);
            textBox3.Text = Convert.ToString(kernel_blur[2, 0]);
            textBox4.Text = Convert.ToString(kernel_blur[0, 1]);
            textBox5.Text = Convert.ToString(kernel_blur[1, 1]);
            textBox6.Text = Convert.ToString(kernel_blur[2, 1]);
            textBox7.Text = Convert.ToString(kernel_blur[0, 2]);
            textBox8.Text = Convert.ToString(kernel_blur[1, 2]);
            textBox9.Text = Convert.ToString(kernel_blur[2, 2]);
        }

        private void buttonSharp_Click(object sender, EventArgs e)
        {
            textBox1.Text = Convert.ToString(kernel_sharp[0, 0]);
            textBox2.Text = Convert.ToString(kernel_sharp[1, 0]);
            textBox3.Text = Convert.ToString(kernel_sharp[2, 0]);
            textBox4.Text = Convert.ToString(kernel_sharp[0, 1]);
            textBox5.Text = Convert.ToString(kernel_sharp[1, 1]);
            textBox6.Text = Convert.ToString(kernel_sharp[2, 1]);
            textBox7.Text = Convert.ToString(kernel_sharp[0, 2]);
            textBox8.Text = Convert.ToString(kernel_sharp[1, 2]);
            textBox9.Text = Convert.ToString(kernel_sharp[2, 2]);
        }

        private void buttonExecute_Click(object sender, EventArgs e)
        {
            Graphics gr = CreateGraphics();
            //テキストボックスからカーネルを読み込む
            try
            {
                kernel[0, 0] = Convert.ToSingle(textBox1.Text);
                kernel[1, 0] = Convert.ToSingle(textBox2.Text);
                kernel[2, 0] = Convert.ToSingle(textBox3.Text);
                kernel[0, 1] = Convert.ToSingle(textBox4.Text);
                kernel[1, 1] = Convert.ToSingle(textBox5.Text);
                kernel[2, 1] = Convert.ToSingle(textBox6.Text);
                kernel[0, 2] = Convert.ToSingle(textBox7.Text);
                kernel[1, 2] = Convert.ToSingle(textBox8.Text);
                kernel[2, 2] = Convert.ToSingle(textBox9.Text);
            }
            catch (System.FormatException ex)
            {
                MessageBox.Show(ex.Message, "入力エラー");
            }
            //原画像にコンボリューションを行う
            Bitmap bmap1 = MakeConvolution(pdata, kernel);
            //コンボリューション結果を左下に表示する
            gr.DrawImage(bmap1, 10, 260, WIDTH, HEIGHT);
        }        
    }
}
