﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace WindowsFormsApp13
{
    public partial class Form1 : Form
    {
        const int WIDTH = 320, HEIGHT = 240;
        Bitmap bmap1 = new Bitmap(WIDTH, HEIGHT);      //原画像用
        Bitmap bmap2 = new Bitmap(WIDTH, HEIGHT);      //エッジ画像用
        Color[,] pdata = new Color[WIDTH, HEIGHT];     //原画像ピクセルのカラー情報
        int threshold;                                 //エッジ検出結果に対するしきい値
        int[,] kernel1 = new int[3, 3];   //カーネルを選択して入れる場所（横方向）
        int[,] kernel2 = new int[3, 3];   //カーネルを選択して入れる場所（縦方向）
        int[,] kernel_p1 = {{-1, 0, 1},   //Prewittカーネル（横方向）                                   
                            {-1, 0, 1},
                            {-1, 0, 1}};
        int[,] kernel_p2 = {{-1,-1,-1},   //Prewittカーネル（縦方向）                                   
                            { 0, 0, 0},
                            { 1, 1, 1}};
        int[,] kernel_s1 = {{-1, 0, 1},   //Sobelカーネル(横方向）                                   
                            {-2, 0, 2},
                            {-1, 0, 1}};
        int[,] kernel_s2 = {{-1,-2,-1},   //Sobelカーネル(縦方向）                                   
                            { 0, 0, 0},
                            { 1, 2, 1}};
        int[,] kernel_l1 = {{-1,-1,-1},   //Laplacianカーネル（両方向）                                   
                            {-1, 8,-1},
                            {-1,-1,-1}};

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            int i, j;

            //原画像を読み込む
            bmap1 = new Bitmap("gent.jpg");
            //原画像を二次元ピクセルデータpdataに変換する
            for (j = 0; j < HEIGHT; j++)
                for (i = 0; i < WIDTH; i++)
                {
                    pdata[i, j] = bmap1.GetPixel(i, j);
                }
            //エッジ画像の周辺を白に設定する
            for (i = 0; i < WIDTH; i++)
            {
                bmap2.SetPixel(i, 0, Color.White);
                bmap2.SetPixel(i, HEIGHT - 1, Color.White);
            }
            for (j = 0; j < HEIGHT; j++)
            {
                bmap2.SetPixel(0, j, Color.White);
                bmap2.SetPixel(WIDTH - 1, j, Color.White);
            }
        }

        private int[,] MakeConvolution(Color[,] pdata, int[,] kernel1, int[,] kernel2)
        {
            int i, j, m, n;
            int r1, g1, b1;
            int r2, g2, b2;
            int d;
            int[,] data = new int[WIDTH, HEIGHT];

            //コンボリューションの実施
            for (j = 1; j < HEIGHT - 1; j++)
                for (i = 1; i < WIDTH - 1; i++)
                {
                    r1 = 0; g1 = 0; b1 = 0;
                    r2 = 0; g2 = 0; b2 = 0;
                    for (n = -1; n <= 1; n++)
                        for (m = -1; m <= 1; m++)
                        {
                            r1 += pdata[i + m, j + n].R * kernel1[m + 1, n + 1];
                            g1 += pdata[i + m, j + n].G * kernel1[m + 1, n + 1];
                            b1 += pdata[i + m, j + n].B * kernel1[m + 1, n + 1];
                            r2 += pdata[i + m, j + n].R * kernel2[m + 1, n + 1];
                            g2 += pdata[i + m, j + n].G * kernel2[m + 1, n + 1];
                            b2 += pdata[i + m, j + n].B * kernel2[m + 1, n + 1];
                        }
                    //二乗値を加算して平方根を求める
                    d = (int)Math.Sqrt(r1 * r1 + g1 * g1 + b1 * b1 + r2 * r2 + g2 * g2 + b2 * b2);
                    data[i, j] = d;
                }
            return data;
        }

        private void Form1_Paint(object sender, PaintEventArgs e)
        {
            Graphics gr = e.Graphics;
            int[,] edata = new int[WIDTH, HEIGHT];   //エッジ強度データ
            int i, j, d;

            //原画像を左上に表示する
            gr.DrawImage(bmap1, 10, 10, WIDTH, HEIGHT);
            //コンボリューションを実行して、エッジ強度データを得る
            edata = MakeConvolution(pdata, kernel1, kernel2);
            //スクロールバーの値を読み込み表示する
            threshold = hScrollBar1.Value;
            label4.Text = Convert.ToString(threshold);
            //エッジ強度データとしきい値でエッジ画像を描く
            for (j = 1; j < HEIGHT - 1; j++)
                for (i = 1; i < WIDTH - 1; i++)
                {
                    if (edata[i, j] > threshold) d = 0;
                    else d = 255;
                    bmap2.SetPixel(i, j, Color.FromArgb(d, d, d));
                }
            //エッジ画像を左下に表示する
            gr.DrawImage(bmap2, 10, 260, WIDTH, HEIGHT);
        }

        //「Prewitt」にチェックが付いた
        private void radioButton1_CheckedChanged(object sender, EventArgs e)
        {        
            kernel1 = kernel_p1;
            kernel2 = kernel_p2;
            Invalidate();
        }

        //「Sobel」にチェックが付いた
        private void radioButton2_CheckedChanged(object sender, EventArgs e)
        {
            kernel1 = kernel_s1;
            kernel2 = kernel_s2;
            Invalidate();
        }

        //「Laplacian」にチェックが付いた
        private void radioButton3_CheckedChanged(object sender, EventArgs e)
        {
            kernel1 = kernel_l1;
            kernel2 = kernel_l1;
            Invalidate();
        }

        private void hScrollBar1_Scroll(object sender, ScrollEventArgs e)
        {
            threshold = hScrollBar1.Value;
            label4.Text = Convert.ToString(threshold);
            Invalidate();
        }
    }
}
