﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace WindowsFormsApp15
{
    public partial class Form1 : Form
    {
        const int WIDTH = 320, HEIGHT = 240;
        Bitmap bmap_src = new Bitmap(WIDTH, HEIGHT);
        Bitmap bmap_gray = new Bitmap(WIDTH, HEIGHT);
        int[,] pdata = new int[WIDTH, HEIGHT];
        int[,] integrate = new int[WIDTH, HEIGHT];

        public Form1()
        {
            InitializeComponent();
        }

        private int[,] GetGrayData(Bitmap bmap)
        {
            int i, j;
            Color color1;
            int r, g, b, y;
            int[,] data = new int[WIDTH, HEIGHT];
            for (j = 0; j < HEIGHT; j++)
                for (i = 0; i < WIDTH; i++)
                {
                    color1 = bmap.GetPixel(i, j);
                    r = color1.R;
                    g = color1.G;
                    b = color1.B;
                    y = (r * 2 + g * 4 + b) / 7;
                    data[i, j] = y;
                }
            return data;
        }

        private int[,] CreateIntegralImage(int[,] pdata)
        {
            int[,] integral = new int[WIDTH, HEIGHT];
            int sum = 0;
            int i, j;
            //一番上の行
            for (i = 0; i < WIDTH; i++)
            {
                sum += pdata[i, 0];
                integral[i, 0] = sum;
            }
            //二番目以降の行
            for (j = 1; j < HEIGHT; j++)
            {
                sum = 0;
                for (i = 0; i < WIDTH; i++)
                {
                    sum += pdata[i, j];
                    integral[i, j] = integral[i, j - 1] + sum;
                }
            }
            return integral;
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            int i, j, y;
            //原画像を読み込む
            bmap_src = new Bitmap("gent.jpg");
            //原画像をグレイピクセルデータにする
            pdata = GetGrayData(bmap_src);
            //ピクセルデータでグレイ画像を生成する
            for (j = 0; j < HEIGHT; j++)
                for (i = 0; i < WIDTH; i++)
                {
                    y = pdata[i, j];
                    bmap_gray.SetPixel(i, j, Color.FromArgb(y, y, y));
                }
            //積分形二次元配列を得る
            integrate = CreateIntegralImage(pdata);
            //コンボボックスの初期設定をする
            comboBox1.SelectedIndex = 2;
            comboBox2.SelectedIndex = 3;
        }

        private int[,] GetThreshold(int[,] integral, int l_width, float k)
        {
            int global_mean, local_mean;
            int i, j, ii, jj;
            int h = l_width / 2;
            int area = l_width * l_width;
            int[,] threshold = new int[WIDTH, HEIGHT];
            //integral用いて、グローバルの平均global_meanを得る
            global_mean = integral[WIDTH - 1, HEIGHT - 1] / (WIDTH * HEIGHT);
            for (j = 0; j < HEIGHT; j++)
                for (i = 0; i < WIDTH; i++)
                {
                    //周辺処理
                    if (i < h) ii = h;
                    else if (i >= WIDTH - h) ii = WIDTH - h - 1;
                    else ii = i;
                    if (j < h) jj = h;
                    else if (j >= HEIGHT - h) jj = HEIGHT - h - 1;
                    else jj = j;
                    //integralを用いて、±hの範囲の平均local_meanを得る
                    local_mean = (integral[ii + h, jj + h] + integral[ii - h, jj - h] - integral[ii + h, jj - h] - integral[ii - h, jj + h]) / area;
                    //寄与率を用いて、meanからthresholdを求める
                    threshold[i, j] = (int)((1 - k) * global_mean + k * local_mean);
                }
            return threshold;
        }

        private void Form1_Paint(object sender, PaintEventArgs e)
        {
            Graphics gr = e.Graphics;

            int X0 = 10, Y0 = 44;
            int X1 = WIDTH + 20, Y1 = Y0;
            int X2 = X0, Y2 = HEIGHT + 54;
            int X3 = X1 + 30, Y3 = Y2;
            int i, j, y;
            int local_width = 32;    //仮設定
            float coef = 0.06F;      //仮設定
            int[,] threshold = new int[WIDTH, HEIGHT];

            //画像を左上に表示する
            gr.DrawImage(bmap_src, X0, Y0);
            //グレイ画像を右上に表示する
            gr.DrawImage(bmap_gray, X1, Y1);
            //コンボボックスを読み取る
            switch (comboBox1.SelectedIndex)
            {
                case 0: local_width = 8; break;
                case 1: local_width = 16; break;
                case 2: local_width = 32; break;
                case 3: local_width = 64; break;
                default: break;
            }
            switch (comboBox2.SelectedIndex)
            {
                case 0: coef = 0F; break;
                case 1: coef = 0.2F; break;
                case 2: coef = 0.4F; break;
                case 3: coef = 0.6F; break;
                case 4: coef = 0.8F; break;
                case 5: coef = 1F; break;
                default: break;
            }
            //積分形二次元配列から、しきい値二次元配列を求める
            threshold = GetThreshold(integrate, local_width, coef);
            //しきい値を画像にする
            Bitmap bmap_thr = new Bitmap(WIDTH, HEIGHT);
            for (j = 0; j < HEIGHT; j++)
                for (i = 0; i < WIDTH; i++)
                {
                    y = threshold[i, j];
                    bmap_thr.SetPixel(i, j, Color.FromArgb(y, y, y));
                }
            //しきい値画像を左下に表示する
            gr.DrawImage(bmap_thr, X2, Y2);
            //ピクセルデータとしきい値で二値画像を生成する
            Bitmap bmap_bin = new Bitmap(WIDTH, HEIGHT);
            for (j = 0; j < HEIGHT; j++)
                for (i = 0; i < WIDTH; i++)
                {
                    if (pdata[i, j] > threshold[i, j]) y = 255;
                    else y = 0;
                    bmap_bin.SetPixel(i, j, Color.FromArgb(y, y, y));
                }
            //二値画像を左下に表示する
            gr.DrawImage(bmap_bin, X1, Y2);
        }
    
        private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            Invalidate();
        }

        private void comboBox2_SelectedIndexChanged(object sender, EventArgs e)
        {
            Invalidate();
        }
    }
}
