﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace WindowsFormsApp20
{
    public partial class Form1 : Form
    {
        int X0 = 60, Y0 = 30;
        int SIZE = 40;

        private delegate double Function(double t, double x, double y);

        public Form1()
        {
            InitializeComponent();
        }

        private void CalculateRungeKutta(Function func1, Function func2, double xinit, double yinit, Color col)
        {
            Graphics g = CreateGraphics();

            double H = 2.5E-8;
            double t, m1, m2, m3, m4, n1, n2, n3, n4;
            double x = xinit;
            double y = yinit;
            float xx, yy, xx_old = 0F, yy_old = 0F;
            Pen pen1 = new Pen(col, 2F);
 
            g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
            for (int i = 0; i < 400; i++)
            {
                t = H * i;
                m1 = H * func1(t, x, y);
                n1 = H * func2(t, x, y);
                m2 = H * func1(t + H / 2, x + m1 / 2, y + n1 / 2);
                n2 = H * func2(t + H / 2, x + m1 / 2, y + n1 / 2);
                m3 = H * func1(t + H / 2, x + m2 / 2, y + n2 / 2);
                n3 = H * func2(t + H / 2, x + m2 / 2, y + n2 / 2);
                m4 = H * func1(t + H / 2, x + m3 / 2, y + n3 / 2);
                n4 = H * func2(t + H / 2, x + m3 / 2, y + n3 / 2);
                x += (m1 + 2 * m2 + 2 * m3 + m4) / 6;
                y += (n1 + 2 * n2 + 2 * n3 + n4) / 6;
                xx = (float)(X0 + i * SIZE * 0.025);
                yy = (float)(Y0 + SIZE * 5 - x * SIZE * 50);
                if (t != 0.0)
                    g.DrawLine(pen1, xx_old, yy_old, xx, yy);
                xx_old = xx;
                yy_old = yy;
            }
        }

        private void Form1_Paint(object sender, PaintEventArgs e)
        {
            Graphics g = e.Graphics;

            int i;
            String string1;
            Pen pen1;
            const double L = 2.0E-5, C = 2.0E-9;
            double E = 0, R = 0, OMEGA = 0;

            Function function1 = (t, x, y) => y;
            Function function2 = (t, x, y) => -(x / C + R * y) / L;
            Function function3 = (t, x, y) => (OMEGA * E * Math.Cos(OMEGA * t) - x / C - R * y) / L;

            //目盛を入れる
            for (i = 0; i <= 10; i++)
            {
                //縦軸
                g.DrawLine(Pens.Black, X0 + i * SIZE, Y0, X0 + i * SIZE, Y0 + SIZE * 10);
                string1 = String.Format("{0}μs", i);
                g.DrawString(string1, Font, Brushes.Black, X0 - 10 + i * SIZE, Y0 + 10 + SIZE * 10);
                //横軸
                g.DrawLine(Pens.Black, X0, Y0 + i * SIZE, X0 + SIZE * 10, Y0 + i * SIZE);
                string1 = String.Format("{0, 4:D}mA", (i - 5) * 20);
                g.DrawString(string1, Font, Brushes.Black, X0 - 10 - SIZE, Y0 - 5 + SIZE * 5 - (i - 5) * SIZE);
            }

            //---- LCR回路に直流を加えた時の過渡現象はこちらを使う -----------
            //不足制動
            E = 10.0; R = 20.0;
            CalculateRungeKutta(function1, function2, 0.0, E / L, Color.Black);
            g.DrawString("不足制動", Font, Brushes.Black, 550, 94);
            pen1 = new Pen(Color.Black, 2F);
            g.DrawLine(pen1, 480, 100, 540, 100);
            //臨界制動
            E = 20.0; R = 200.0;
            CalculateRungeKutta(function1, function2, 0.0, E / L, Color.DimGray);
            g.DrawString("臨界制動", Font, Brushes.Black, 550, 134);
            pen1 = new Pen(Color.DimGray, 2F);
            g.DrawLine(pen1, 480, 140, 540, 140);
            //過制動
            E = 50.0; R = 500.0;
            CalculateRungeKutta(function1, function2, 0.0, E / L, Color.DarkGray);
            g.DrawString("過制動", Font, Brushes.Black, 550, 174);
            pen1 = new Pen(Color.DarkGray, 2F);
            g.DrawLine(pen1, 480, 180, 540, 180);
            //----------------------------------------------------------------

            //---- LCR回路に交流を加えた時の過渡現象はこちらを使う -----------
            //E = 1.0; R = 15.0;
            ////共振周波数
            //OMEGA = 5.0E+6;
            //CalculateRungeKutta(function1, function3, 0.0, E / L, Color.Black);
            //g.DrawString("共振周波数", Font, Brushes.Black, 550, 94);
            //pen1 = new Pen(Color.Black, 2F);
            //g.DrawLine(pen1, 480, 100, 540, 100);
            ////共振周波数x0.8
            //OMEGA = 4.0E+6;
            //CalculateRungeKutta(function1, function3, 0.0, E / L, Color.DimGray);
            //g.DrawString("共振周波数x0.8", Font, Brushes.Black, 550, 134);
            //pen1 = new Pen(Color.DimGray, 2F);
            //g.DrawLine(pen1, 480, 140, 540, 140);
            ////共振周波数x1.2
            //OMEGA = 6.0E+6;
            //CalculateRungeKutta(function1, function3, 0.0, E / L, Color.DarkGray);
            //g.DrawString("共振周波数x1.2", Font, Brushes.Black, 550, 174);
            //pen1 = new Pen(Color.DarkGray, 2F);
            //g.DrawLine(pen1, 480, 180, 540, 180);
            //----------------------------------------------------------------
        }
    }
}
