﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace WindowsFormsApp21
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void Draw1DWaveform(float[] data, int n, int x0, int y0)
        {
            Graphics g = CreateGraphics();
            g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
            int x, y, y_old = 0;
            for (x = 0; x < n; x++)
            {
                y = y0 - (int)(50 * data[x]);
                if (x != 0)
                {
                    g.DrawLine(Pens.Black, x0 + (x - 1), y_old, x0 + x, y);
                    y_old = y;
                }
                y_old = y;
            }
        }

        private void Form1_Paint(object sender, PaintEventArgs e)
        {
            Graphics g = e.Graphics;

            int X0 = 25, X1 = 550;
            int Y0 = 100, Y1 = 260, Y2 = 420;
            int N = 512;
            int CYCLE = 4;
            float S2N = (float)(Math.Sqrt(2.0 / N));
            float SN = (float)(Math.Sqrt(N));
            int UMAX = 128;
            int i, x, u;
            float[] source = new float[N];
            float[] dct = new float[N];
            float[] idct = new float[N];
            float[,] cs = new float[N, N];

            //グラフの枠を描く
            for (i = 0; i < 3; i++)
            {
                g.DrawRectangle(Pens.Black, X0, Y0 - 75 + 160 * i, 512, 150);
                g.DrawLine(Pens.Black, X0, Y0 + 160 * i, X0 + 512, Y0 + 160 * i);
            }
            //原波形（矩形波）を生成する
            for (x = 0; x < N; x++)
            {
                if ((x + N / CYCLE / 4) % (N / CYCLE) > N / (CYCLE * 2)) source[x] = -1.0F;
                else source[x] = 1.0F;
            }
            //原波形を描く
            Draw1DWaveform(source, N, X0, Y0);
            g.DrawString("原波形", Font, Brushes.Black, X1, Y0 - 5);
            //コサイン関数表を作成する
            for (x = 0; x < N; x++)
                for (u = 0; u < N; u++)
                    cs[x, u] = (float)(Math.Cos((2 * x + 1) * u * Math.PI / (2 * N)));
            //DCTを実行して描画する
            dct[0] = 0.0F;
            for (x = 0; x < N; x++)
                dct[0] += source[x];
            dct[0] = dct[0] / SN;
            g.DrawLine(Pens.Black, X0, Y1, X0, Y1 - (int)(3 * dct[0]));
            for (u = 1; u < N; u++)
            {
                dct[u] = 0.0F;
                for (x = 0; x < N; x++)
                    dct[u] += source[x] * cs[x, u];
                dct[u] *= S2N;
                g.DrawLine(Pens.Black, X0 + u, Y1, X0 + u, Y1 - (int)(3 * dct[u]));
            }
            g.DrawString("DCTデータ", Font, Brushes.Black, X1, Y1 - 5);
            //IDCTを実行して描画する
            for (x = 0; x < N; x++)
            {
                idct[x] = 0.0F;
                for (u = 1; u <= UMAX; u++)
                    idct[x] += dct[u] * cs[x, u];
                idct[x] = idct[x] * S2N + dct[0] / SN;
            }
            Draw1DWaveform(idct, N, X0, Y2);
            g.DrawString("IDCT波形", Font, Brushes.Black, X1, Y2 - 5);
        }
    }
}
