﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace WindowsFormsApp22
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void Draw2DImage(int[,] image, int n, int x0, int y0)
        {
            Graphics g = CreateGraphics();
            int x, y;
            int d;
            Brush brush1;
            for (y = 0; y < n; y++)
                for (x = 0; x < n; x++)
                {
                    d = image[x, y];
                    brush1 = new SolidBrush(Color.FromArgb(d, d, d));
                    g.FillRectangle(brush1, x0 + x * 2, y0 + y * 2, 2, 2);
                }
        }

        private void Draw2DDCT(float[,] dctdata, int n, int x0, int y0)
        {
            Graphics g = CreateGraphics();
            int x, y;
            int d;
            Brush brush1;
            for (y = 0; y < n; y++)
                for (x = 0; x < n; x++)
                {
                    d = 128 + (int)(dctdata[x, y] / 4);
                    if (d > 255) d = 255;
                    else if (d < 0) d = 0;
                    brush1 = new SolidBrush(Color.FromArgb(d, d, d));
                    g.FillRectangle(brush1, x0 + x * 2, y0 + y * 2, 2, 2);
                }
        }

        private void Form1_Paint(object sender, PaintEventArgs e)
        {
            Graphics g = e.Graphics;
            int X0 = 10, Y0 = 10;
            int X1 = 276, Y1 = 276;
            int N = 128;
            int[,] source = new int[N, N];
            float[,] dct = new float[N, N];
            float[,] dct_limited = new float[N, N];
            int[,] idct = new int[N, N];
            float[,] cc = new float[N, N];
            float[,] cs = new float[N, N];
            int x, y, xx, yy, u, v;
            float sum;

            //原画像を生成する
            for (x = 0; x < N; x++)
                for (y = 0; y < N; y++)
                {
                    xx = (x + N / 8) % (N / 2);
                    yy = (y + N / 8) % (N / 2);
                    if ((xx > N / 4 && yy <= N / 4) || (xx <= N / 4 && yy > N / 4))
                        source[x, y] = 192;
                    else source[x, y] = 64;
                }
            //原画像を描画する
            Draw2DImage(source, N, X0, Y0);
            //DCTのための係数の準備
            for (u = 0; u < N; u++)
                for (v = 0; v < N; v++)
                {
                    if (u + v == 0) cc[u, v] = 0.5F;
                    else if (u * v == 0) cc[u, v] = 1 / (float)(Math.Sqrt(2.0F));
                    else cc[u, v] = 1F;
                }
            for (x = 0; x < N; x++)
                for (u = 0; u < N; u++)
                    cs[x, u] = (float)(Math.Cos((2 * x + 1) * u * Math.PI / (2.0F * N)));
            //DCTを実施する
            for (u = 0; u < N; u++)
                for (v = 0; v < N; v++)
                {
                    sum = 0F;
                    for (x = 0; x < N; x++)
                        for (y = 0; y < N; y++)
                            sum += source[x, y] * cs[x, u] * cs[y, v];
                    dct[u, v] = cc[u, v] * sum * 2 / N;
                }
            //DCTデータを描画する
            Draw2DDCT(dct, N, X1, Y0);
            //IDCTを実施する（u + vの最大値をN / 2 - 1にして）
            for (x = 0; x < N; x++)
                for (y = 0; y < N; y++)
                {
                    sum = 0F;
                    for (u = 0; u < N / 2; u++)
                        for (v = 0; v < N / 2; v++)
                        {
                            sum += cc[u, v] * dct[u, v] * cs[x, u] * cs[y, v];
                            dct_limited[u, v] = dct[u, v];
                        }
                    idct[x, y] = (int)(sum * 2 / N + 0.5F);
                    if (idct[x, y] > 255) idct[x, y] = 255;
                    else if (idct[x, y] < 0) idct[x, y] = 0;
                }
            //制限を加えたDCTを描画する
            Draw2DDCT(dct_limited, N, X0, Y1);
            //IDCTを描画する
            Draw2DImage(idct, N, X1, Y1);
        }
    }
}
