﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace WindowsFormsApp23
{
    public partial class Form1 : Form
    {
        int X0 = 210, Y0 = 210;
        float cr, ci, step;
        float cr_old, ci_old, step_old;
        int count_max, color_number;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            //初期設定をする
            cr = -0.75F;
            ci = 0.0F;
            step = 0.00625F;
            count_max = 256;
            color_number = 16;
            textBox1.Text = cr.ToString();
            textBox2.Text = ci.ToString();
            textBox3.Text = step.ToString();
            textBox4.Text = count_max.ToString();
            textBox5.Text = color_number.ToString();
        }

        private Color CountToColor(int n, int color_base)
        {
            if (n < 0) return Color.Black;   //発散しない場合は黒で返す
            int d = n % color_base;
            d *= (256 / color_base);
            int h = d / 43;
            int r = 0, g = 0, b = 0;         //仮設定が必要
            switch (h)
            {
                case 0: r = 255;           g = d * 6;         b = 0;             break;  //d=0～42(赤→黄)
                case 1: r = (85 - d) * 6;  g = 255;           b = 0;             break;  //d=43～85(黄→緑)  
                case 2: r = 0;             g = 255;           b = (d - 86) * 6;  break;  //d=86～128(緑→シアン)
                case 3: r = 0;             g = (171 - d) * 6; b = 255;           break;  //d=129～171(シアン→青)
                case 4: r = (d - 172) * 6; g = 0;             b = 255;           break;  //d=172～214(青→マゼンタ)
                case 5: r = 255;           g = 0;             b = (255 - d) * 6; break;  //d=215～255(マゼンタ→赤)
            }
            return Color.FromArgb(r, g, b);
        }

        private void Form1_Paint(object sender, PaintEventArgs e)
        {
            Graphics g = e.Graphics;

            int x, y;
            double c_real, c_imag, z_real, z_imag, z_real2, z_imag2, value;
            int count;
            Brush brush1;

            //テキストボックスから各種データを読込む
            try   //try～catch形式にしないと、実行時にエラーになる
            {
                cr = float.Parse(textBox1.Text);
                ci = float.Parse(textBox2.Text);
                step = float.Parse(textBox3.Text);
                count_max = int.Parse(textBox4.Text);
                color_number = int.Parse(textBox5.Text);
            }
            catch
            {
            }
            //Mandelbrot図形を描く
            for (x = -200; x <= 200; x++)
            {
                c_real = x * step + cr;
                for (y = -200; y <= 200; y++)
                {
                    c_imag = y * step + ci;
                    z_real = 0.0F;
                    z_imag = 0.0F;
                    count = 0;
                    do
                    {
                        z_real2 = z_real * z_real;
                        z_imag2 = z_imag * z_imag;
                        z_imag = 2.0F * z_real * z_imag + c_imag;   //z_realより先に求めること
                        z_real = z_real2 - z_imag2 + c_real;
                        value = z_real2 + z_imag2;
                        count++;
                        if (count > count_max)   //最大繰返し回数でも発散しない
                        {
                            count = -1;
                            break;
                        }
                    } while (value < 4.0F);    //発散判定の条件
                    brush1 = new SolidBrush(CountToColor(count, color_number));
                    g.FillRectangle(brush1, X0 + x, Y0 - y, 1, 1);
                }
            }
        }

        private void Form1_MouseClick(object sender, MouseEventArgs e)
        {
            Rectangle image_area = new Rectangle(X0 - 200, Y0 - 200, 400, 400);
            //描画範囲であれば
            if (image_area.Contains(e.X, e.Y))
            {
                //一つ前の設定に戻す準備のために、それまでのデータを保存
                cr_old = cr;
                ci_old = ci;
                step_old = step;
                //クリックした場所が中心に来るように移動
                cr += (e.X - X0) * step;
                ci -= (e.Y - Y0) * step;
                //右クリックなら５倍に拡大する
                if (e.Button == MouseButtons.Right)
                    step /= 5.0F;
                //テキストボックスへ各種データを書込む
                textBox1.Text = cr.ToString();
                textBox2.Text = ci.ToString();
                textBox3.Text = step.ToString();
                Invalidate();
            }
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            //テキストボックスへ元の各種データを書戻す
            textBox1.Text = cr.ToString();
            textBox2.Text = ci.ToString();
            textBox3.Text = step.ToString();
            textBox4.Text = count_max.ToString();
            textBox5.Text = color_number.ToString();
            Invalidate();
        }

        private void buttonOK_Click(object sender, EventArgs e)
        {
            Invalidate();
        }

        private void buttonUndo_Click(object sender, EventArgs e)
        {
            //一つ前の設定に戻し、テキストボックスへ書込む
            cr = cr_old;
            ci = ci_old;
            step = step_old;
            textBox1.Text = cr.ToString();
            textBox2.Text = ci.ToString();
            textBox3.Text = step.ToString();
            Invalidate();
        }

        private void buttonReset_Click(object sender, EventArgs e)
        {
            //初期設定に戻し、テキストボックスへ書込む
            cr = -0.75F;
            ci = 0.0F;
            step = 0.00625F;
            count_max = 256;
            color_number = 16;
            textBox1.Text = cr.ToString();
            textBox2.Text = ci.ToString();
            textBox3.Text = step.ToString();
            textBox4.Text = count_max.ToString();
            textBox5.Text = color_number.ToString();
            Invalidate();
        }        
    }
}
