﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO;                //Pathのために追加
using System.Drawing.Imaging;   //ImageFormatのために追加


namespace WindowsFormsApp7
{
    public partial class Form1 : Form
    {
        String fileName;                 //原画像ファイル名、上書きに使用
        Bitmap bmap0, bmap1, bmap2;      //原画像、モノクロ画像、反転画像
        Boolean image_flag = false;      //原画像を読み込んだらtrue
        int processType;                 //0 --- 原画像、1 --- モノクロ化、2 --- 色反転

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Paint(object sender, PaintEventArgs e)
        {
            Graphics gr = e.Graphics;
            int X0 = 10, Y0 = 30;     //画像の左上の座標
            Color color0, color1, color2;
            int r0, g0, b0, r2, g2, b2, y;

            if (image_flag)
            {
                switch (processType)
                {
                    case 0:
                        gr.DrawImage(bmap0, X0, Y0);
                        break;
                    case 1:
                        bmap1 = new Bitmap(bmap0.Width, bmap0.Height);
                        for (int j = 0; j < bmap0.Height; j++)
                            for (int i = 0; i < bmap0.Width; i++)
                            {
                                color0 = bmap0.GetPixel(i, j);
                                r0 = color0.R;
                                g0 = color0.G;
                                b0 = color0.B;
                                y = (r0 * 2 + g0 * 4 + b0) / 7;
                                color1 = Color.FromArgb(y, y, y);
                                bmap1.SetPixel(i, j, color1);
                            }
                        gr.DrawImage(bmap1, X0, Y0);
                        break;
                    case 2:
                        bmap2 = new Bitmap(bmap0.Width, bmap0.Height);
                        for (int j = 0; j < bmap0.Height; j++)
                            for (int i = 0; i < bmap0.Width; i++)
                            {
                                color0 = bmap0.GetPixel(i, j);
                                r2 = 255 - color0.R;
                                g2 = 255 - color0.G;
                                b2 = 255 - color0.B;
                                color2 = Color.FromArgb(r2, g2, b2);
                                bmap2.SetPixel(i, j, color2);
                            }
                        gr.DrawImage(bmap2, X0, Y0);
                        break;
                    default:
                        break;
                }
            }
        }

        private void menuOpen_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.Title = "画像の読み込み";
            ofd.Filter = "JPGファイル|*.jpg|PNGファイル|*.png|GIFファイル|*.gif|BMPファイル|*.bmp";
            if (ofd.ShowDialog() == DialogResult.OK)
            {
                fileName = ofd.FileName;
                Bitmap bmap = new Bitmap(fileName);
                bmap0 = new Bitmap(bmap);
                bmap.Dispose();    //これで上書きが可能になる　
                image_flag = true;
                processType = 0;
                Invalidate();
            }
            ofd.Dispose();
        }

        private void menuSave_Click(object sender, EventArgs e)
        {
            if (image_flag == false)
            {
                MessageBox.Show("画像がありません", "エラー");
                return;
            }
            ImageFormat imf = ImageFormat.Jpeg;  //仮設定
            String ext = Path.GetExtension(fileName);
            if (ext == ".jpeg") imf = ImageFormat.Jpeg;
            else if (ext == ".png") imf = ImageFormat.Png;
            else if (ext == "gif") imf = ImageFormat.Gif;
            else if (ext == "bmp") imf = ImageFormat.Bmp;
            Bitmap bmap_save = new Bitmap(bmap0.Width, bmap0.Height);
            if (processType == 0) bmap_save = bmap0;
            else if (processType == 1) bmap_save = bmap1;
            else if (processType == 2) bmap_save = bmap2;
            bmap_save.Save(fileName, imf);
        }

        private void menuSaveAs_Click(object sender, EventArgs e)
        {
            if (image_flag == false)
            {
                MessageBox.Show("画像がありません", "エラー");
                return;
            }
            ImageFormat imf = ImageFormat.Jpeg;  //仮設定
            SaveFileDialog sfd = new SaveFileDialog();
            sfd.Title = "画像の保存";
            sfd.Filter = "JPGファイル|*.jpg|PNGファイル|*.png|GIFファイル|*.gif|BMPファイル|*.bmp";
            sfd.OverwritePrompt = true;
            sfd.AddExtension = true;
            if (sfd.ShowDialog() == DialogResult.OK)
            {
                String ext = Path.GetExtension(sfd.FileName);
                if (ext == ".jpeg") imf = ImageFormat.Jpeg;
                else if (ext == ".png") imf = ImageFormat.Png;
                else if (ext == "gif") imf = ImageFormat.Gif;
                else if (ext == "bmp") imf = ImageFormat.Bmp;
                Bitmap bmap_save = new Bitmap(bmap0.Width, bmap0.Height);
                if (processType == 0) bmap_save = bmap0;
                else if (processType == 1) bmap_save = bmap1;
                else if (processType == 2) bmap_save = bmap2;
                bmap_save.Save(sfd.FileName, imf);
            }
            sfd.Dispose();
        }

        private void menuMono_Click(object sender, EventArgs e)
        {
            if (image_flag)
            {
                processType = 1;
                Invalidate();
            }
        }

        private void menuRev_Click(object sender, EventArgs e)
        {
            if (image_flag)
            {
                processType = 2;
                Invalidate();
            }
        }

        private void menuUndo_Click(object sender, EventArgs e)
        {
            if (image_flag)
            {
                processType = 0;
                Invalidate();
            }
        }
    }
}
